# Phatch - Photo Batch Processor
# Copyright (C) 2007-2008 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/
#
# Phatch recommends SPE (http://pythonide.stani.be) for editing python files.

# Embedded icon is designed by Igor Kekeljevic (http://www.admiror-ns.co.yu).

from core import ct, models
from core.translation import _t

from core.lib import gps

class Action(models.Action):
    """Defined variables: <filename> <type> <folder> <width> <height>"""
    
    label       = _t('Geotag')
    author      = 'Robin Mills'
    email       = 'robin@clanmills.com'
    version     = '0.1'
    tags        = [_t('metadata')]
    __doc__     = _t('Geotag an image file')
    cache       = True
    valid_last  = True
    
    def interface(self,fields):
        fields[_t('GPS Data (gpx)')]= self.GeoReadFileField(' ')
        fields[_t('GPS Report (csv)')]= self.CsvFileField(' ')
        fields[_t('Time Shift (seconds)')] = self.IntegerField(0,
            choices=[-60,0,60])
        
    def apply(self,photo,setting,cache):
        #get info
        info                    = photo.get_info() #read-only

        # get timeshift
        # construct timedict if necessary
        if not ('gps_timedict' in cache):
            cache['gps_timeshift']  = \
                self.get_field('Time Shift (seconds)',info)
            gpx_file = self.get_field('GPS Data (gpx)',info)
            csv_file = self.get_field('GPS Report (csv)',info).strip()
            if csv_file:
                csv_file            = open(csv_file,'w')
                gps.write_header(csv_file)
            cache['gps_report']     = csv_file
            cache['gps_timedict' ]  = gps.read_gpx(gpx_file)      
        
        # get the geoTags 
        gps_data = gps.get_metadata(info['Exif.Image.DateTime'],  
            cache['gps_timedict'], cache['gps_timeshift'],  
            info['path'], cache['gps_report'])

        photo.metadata.update(gps_data)
        photo.info.update(gps_data) #writable
        
        return photo
    
    def is_done(self,photo):
        """Method used for resuming when a batch was interrupted.
        For metadata there is no way to know if this image has been done
        already, so return False by default."""
        return False
        
    def is_overwrite_existing_images_forced(self):
        """Always force overwrite as we want to store the tags
        in existing images."""
        return True
    
    ##
    # The beautiful icon is thanks to the work of the Geotag Icon Project
    # http://www.geotagicons.com/
    #
    icon = \
'x\xda\x01N\n\xb1\xf5\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\x00\
\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\x08\
\x08|\x08d\x88\x00\x00\n\x05IDATh\x81\xb5\x9a{l\x14\xd5\x1e\xc7?\xf3\xd8\xed\
v\xbb\xa5\x0fZ\xfa\xa2X\xdb\xe6j\xee\xa5H\x05\x8d\xb1&\x04\xd1\x90`"\xb9^\
\xa2\x82\x11,%\x80\xad\x045\xc6DM\x1a\x1b\x1eF\x85\x84\xa0\x98\x08!5\x12\x15\
c06FCb\xbc\x86\xe4\xf6\x0f\x15\xe4VEy\xd8\x82\x94\x96\xbe\xa4\xed\xb6\xb4tgg\
\xe6\xfe\xb13\xd3\xb3\xb3\xb3\xdb-z\x7f\xc9\xc9\x9c93s\xce\xf7{~\x8f\xf3;3#I\
\xb2\x82(\xda\xc4us\xf0\xd4).\xb5\xb5\xd1\xdf\xde\xceDw7h\x1aH\x12\xff\x0f1\
\x93^0\x91|>2\xcb\xcb)\xae\xab\xa3b\xf5j\n\x97,\xc1\x17\xcc\x8a\x03"\x89\x04\
\xc2\x97.\x9a\xa7v\xef\xa6\xab\xb5\x95\xec\xa9)r\x80\x0c@\xfe\x13\xa0$\xe1<\
\x9d)\x10\x9f5\x80\x1b\xc0(0\x9e\x91AU}=K^~\x999\x15\xb7:]9\x04\x86:\xfek\
\xfe\xbb\xbe\x9e\xe8\xc9\x93\xcc\x07\xfc\xc0u`\xc2\xeaHr\x15\x04b\x92p\x94\\\
\xed"h\xd9\xa3\xcd\xdd\xaf\xd8\x97b\x15,\x12=\x80\x7f\xe9R\xeeom\xa5\xe0\x8e\
\xc5\x92C \xdc\xd5i~\xb9z5JG\x07\x15\xc0\x08\xd0\x0fL\t\x03\xcbB]\x04*\xb9\
\xda\xc5c\xb2\xfb\xdd\x93\xe0n\x97\x88\x1fS!f\tQ\xa0\x0b0\xef\xb8\x83Umm\xcc\
\xa9\xac\x92Tm\xe2\xba\xf9\x9f\xed\xdb\xd1;:\xa8\x02\xfa,\xf0\x12\xa0\xa6\
\x18\xc0k@/\rx\x11\x81\x98\xa9(\x8a\x82\xa9\xeb\xcel\'\xeb\xdb\x044\xc0\x07T\
\x02\xe7::8\xf5\xdakh\x13\xd7My\xe8\xf4i:\x0f\x1ff\x010l\x81W\x04\xf0\xa2JI1\
H\xb2\xfb\xdcm\x00\x81\xf2r\x16\x1e9\xc2\xdd\xdf\x7fOuK\x0br(\x84\xe1\xd1\
\xbf[\xab\x11\x0bW\x19\xd0y\xf80C\xa7O\xa3\xfc37\xf7\xd5H{;\xf9\xc0eb\xf6\
\xaex\xcc\xa8\x97\t\x88\xaa\xf6\xd2\x82\xd7u\x14\x85\xbf\x1f<H\xf1\xda\xb5\
\xf8KJ\xc8]\xb6\x8c@Q\x11\xd7\xbe\xf9\x06"\x918M\xc8B1\xac\xa2\x03\x99\xc0\
\xb8a\xa0\xe7\xe6\xa2\x0e\xb4\xb7\x93C\xccam\x86\xe9\x98\x0c\x1e\x1aJ\xe5\
\xc421SP\xfd~\xb2\x97.E\x94\xa2\x86\x06\x90$~{\xf6Y\xcc\xb11\xc7qE\x132\xad\
\xbaa\xf5\x95\x03\x0c\xb4\xb7\xa3\xac\x96\xa4W\xf3\xc2a&,\x12j\x92Yv\x83WH$\
\x92L\xfdb\xdd\x88F\tVW3\xe7\xee\xbb\xe3H\x84jk\t\x14\x173\xf2\xf5\xd7\x98\
\x82&l\xf0X\xe0MA#\xd7$\t\xe5_YY\xaf\xe6\x8c\x8f3I,\xe6\xaa\xae\x81\xbd\x9c8\
UTr\x03\xf62\xab\x91o\xbf%P\\Lh\xf1\xe28\x12Y\xb5\xb5\x04\xca\xca\x18\xb5H\
\xd83n\n\xc5\x0e\xe9\x06\x10\xce\xcaBF\x92f\xb4uQ\x9d"(\xb7\xbdB"Qw\x9b\x04\
\x98\xd7\xaeaLN\xe2%\x05\x1b6Py\xe0\x00Rv6Q\xab\xcdM\xc0Y\xec$iz\x91\xfd\xab\
\xa3\x8d\x17q\x13\x90|>\xfev\xe0\x00\xc5\x1b7z\x12\x00\x98\xfb\xe4\x93\xdc\
\xba\x7f?\x84Bh.\x02\xee\xd4C\x15\t\xa4r\xded\xd7\xdc\xa6\x94\x8c\xbc\x01\
\xc8\x81\x00\xd5{\xf7R\xd2\xd8\x98\x14\xbc-\x05O=\x85\xa1\xeb\\x\xee9\xa2ccN\
\xdf\xa2C\xdb\xe3&\x1d\xf4f\xa3\x8d\xfb\x1e\x07\xfc\x9e=\x94\xa6\x01\xde\x96\
y\r\rT\xef\xdb\x87\x9c\x9d\x8d\x8e\xb7\x16T\xf1\x01/\xf0\xa9\xa2\x8dW\x9b[[\
\x06\xa0\x04\x02T\xbf\xf1\x06\xa5MMi\x83\xb7\xa5\xc82\xb5s\xdb\xb7c\x8c\x8f\
\'\x10\x90\xc5\x8a\xd7\xecy\x01uG\x1b\x93\xd8\x02\xa3\xb9J\x14a\xe6\xb7m\x9b\
5x\x91D\xe9\xa6Mh\xe0\xac\xd8\xb6$\xd5\xc0L\xb9\x8dML\x03\x82\x99\x99d\xddy\
\'\xa1\x9a\x1a\x02\x85\x85\x00Dz{\x19\xff\xe9\'J\xd6\xaf\xbf\xa9\x99wK\xce}\
\xf7a\xec\xdb\xe7\x04\x03\x1bC\x9c\x13\xbbIx\xb5\xc3\xb4]\xfbB!\xe6\xaf_O\
\xf9\xc6\x8d\x04++\x91\x82Ad\x9f\x0f\x00#\x12\xc1\x98\x98@\xcd\xcb\x9b\x15\
\xd0H_\x1f\xbf=\xff<\xf9\x0f>Hq}}\xacQ\xd7\x19\xfc\xe2\x8b8G\xb6%N\x03\xa92O\
1\xda\x18\x92D\xce\xe2\xc5\xdc\xf6\xe6\x9b\xe4\xdf\x7f?^\xbb59\x10@\x0e\x04f\
\x05^\x1f\x1e\xe6\xc2\xd6\xad\xf4\xb5\xb5q\xed\xf8q&:;\xc9^\xb2\x84?\x8e\x1f\
\xa7\xf7\xfd\xf7\x91\x89\x99\xaa(\x9ea\xd4>\xf72!\x13\x90d\x99\x92\xb5k\xc9_\
\xb1bV\x00SIth\x88s\x9b61\xd0\xd6\x86\x1f\x88\x0e\x0fsq\xd7.g\xf1R\x88\x8fB)\
\xc3h\xaah\xa3\x00\xb2\xaesi\xdf>\xfa\x8f\x1e\xfdK\xc0k\x03\x03\x0ex\x1b\xa8\
\x18,T\x12Wc[T\x98N\x90\xdc6&F\x1b\x9b\xa4]\xd7z{9\xdb\xd8\x08\xa6I\xd1\xda\
\xb57\r>28\xc8\x85\xcd\x9b\xe9oks\xcc\xc1N\x17l\xa0b]ls\x08\xd8"\x01\xb2\xa2\
 gd\x80\xa6aj\x1aQ\xa6g^\x15\xee\xf3c\xd9lS\x13\x92i2o\xdd\xbaY\x83\xd7\x06\
\x079\xbfu+\x03\x16x\xaf\x94A\x04\x9cr!\x93\x889\xc8\xed/\xbeH\xe9\xbau\x18\
\x91\x08\xfa\xe8(\x13\xbf\xff\xce\xf8\x8f?2\xf6\xddw\\\xff\xe1\x07\xb4\xc9Ig\
0\x1f\xd3\x8eg\xca2E\x8f?>;\xf0\x9b73\xf8\xd9gN\x7fn\xa0n\xf0\x10\x1fB\x1d\
\x02\xa2\xad\x07++\xc9^\xb8\xd0\x19(\x1f\xc041\xa2Q&~\xfe\x99\x81c\xc7\x18\
\xfc\xf4S&\xcf\x9f\xc7\xd0\xf5\xd8\xe0cctm\xd9\x82\xac\xeb\x14>\xf1DZ\x04\
\xf4p\x98\xd1\x13\'\x12r\x9c\x99\x8a\x98^\x83e\xd2"+3\x12I\x1cM\x92\x90}>B\
\xb5\xb5T\xee\xdcI\xed\x89\x13T\xb6\xb4\x90YQ\x11[m\x013\x1c\xa6\xab\xa9\x89\
\xa1\x0f>H\x8b\x80:w.\xc1\x9a\x9a\x84\x1c\xc7m&n\xcd\x88m\x0e\x01\x07gZC\x83\
\xbf\xb0\x90\x05\xaf\xbc\xc2?\xda\xda(z\xf4Q\x94\xeclT\xc0\x18\x1d\xe5\xd2\
\xb6m\\K#:I\xaa\x8aZP\xe0\t\xd8\xcb\x0f\xdc>\x11G\xc0\x9d\xdf\xa4+Y\x8b\x16q\
\xfb\xd1\xa3\x04\x17-r|\xc2\x18\x1e\xa6\xab\xb1\x91\xa1\x8f>\x9a\xf1yC\xd7\
\xe3\xcc\xc2\xf0\xa8\x8b&#^K\xba\x0e\xccVz^\x7f\x9d\xa9_~\x89[\'\xa2\xc3\xc3\
t662\xf8\xe1\x87I\x9f3#\x11\xb4\xabWcu\x0f\xa0\x10\xaf\t\xf7\xd6\xd2\x96\xb8\
(4[\x02Wv\xec\xe0JK\x0b\x8a\xae;I\x96=;\xfa\xc8\x08][\xb7\x82,S\xe8\x11\x9d"\
\xfd\xfd\x84\x7f\xfd\x15<\x00\xdbm^\xe6\xe5iB\x9e\')\xc4\xd4u\xae\xec\xd8Aws\
3\xb2\x05\xdeV\xb7\x9c\x97\x87\xa9(H\x8066\xc6\xf9-[\x18\xf0p\xec?\xbe\xfa\
\x8a)k\xa7\xe5e>^f\xe3\x8e@\t\x98\xd3\xd1\x80i\x18\xf4\xec\xde\xcd\xe5\xe6f\
\xe7\xe5\xabA,\xad\xcey\xf8an\xfb\xfcs\xb2\xee\xb9\x07\xcd\xea/\x1a\x0es\xb6\
\xa9\x89~\x81Dtt\x94\xee\x83\x07\x1d\xf0\xb8\x8e3\xb5\xc5\x11\x10C\xa8\x98*$\
\x93\x9e]\xbb\xb8\xdc\xdc\xec\xbc\x003-\xf0\xb9\xcb\x97S\xf1\xd6[d\xd7\xd5\
\xc5\xf6\xbc\x81\x00\xbauOtt\x94_\x9fy\x86\x81c\xc7\x00\xb8\xb8{7\xe13g\x9c\
\xe7\xbd\xcc\xc6\x0e\xaf\xee\x08\xe4\xde\xd4\xcf\xca\x84l\xb3Q\x99\xce\x0e#@\
\xde\x03\x0fP}\xe4\x08\x19\x0b\x16\x00\x90\xff\xc8#\xe4._\x8e\xcet&\xa9\x8d\
\x8cp\xb6\xa9\x89\xb3[\xb6p\xf9\x9dw\xe2|\xc6+\xda\xe0:w\x9bR\x02f1\x99\xf3\
\x04\xdf\xd2\x12sX\x17\xf8\xdc\x95+\xa9jm\xc5_V6\xddi @\xc9\x86\rH\x99\x99q\
\xe9\xf0T\x7f?\x97\x0e\x1eD\x1f\x1f\x07\x0f\xa0^\t\\\xb2\x05.\x81\x00\xc9\
\x08\x98&W\xf7\xee\xa5{\xe7Nd]wl~\n\xc8[\xb9\x92\xaaC\x87\xf0\xcf\x9f\x9f\
\xf0X\xfeC\x0f\x11\xac\xaa\x8a[i\xc5\x01\xc5\xecr\xa6\xc5,\x15\t\'\x95pvcr\
\xbc!i}}\x0c}\xfc1D\xa3\x0e\xf8\x08\x90\xb7b\x05\xd5\x87\x0e\x91Q^\xeeE\x1b%\
\x14"\xc72\xa3d\xf9L:\xd1&\xd93q\x04l\x91\x00sj\x8a\xb86\xbf\x1f5\x14r\x9cu\
\xca\x02_u\xf80\xfe$\xe0m\xc9[\xb6\x0cSQ\xd2\xcem\xd2Y\x07\xc4#\x80\x8ai\xc6\
1\x998s\x06C\xd3\x9c\xcd\xb9:w.\xa5/\xbc@t|\x9c\xa9\xde^\xf2\xef\xbd\x97\x8a\
={\xf0[\x0e\x9bJB55\xa8\x99\x99\x8e\xcd{\x81soT\xc4\xba\x97o\xc4\xad\x05\xa6\
\x89\x8a\xaa:\xef\xdcU`\xe4\xcb/\xd1z{\xc9\xb8\xe5\x16\xa7\xc3\x9cU\xab\xb8\
\xad\xa6\x86H_\x1f\xc1\x85\x0b\x9133g\x04\x0f\x90QZJFI\tc\x17.81?\x99\xc3z\
\x95d\xa6d\x07\x11T\x159XV\xc6\r\xa6w\\S==\xf4\xbf\xfdv\x02\x18\x7fy9\xa1\
\xbb\xeeJ\x1b<\xc4\xa2\x91RXx\xd3\xd1&\x19\xe1Lb\xa6\x9cYV\x86<\xaf\xae\x8eQ\
\x81\x99\x02\xf4\xed\xdf\xcf\xd0{\xef\xa5\r4\xa9H\x12r0\xe8\t\xd2M(\x1d\xf0Q\
b[\xd9,b\x9f]\xe7\xd5\xd5!W\xadY\xc3\xb8\xaa2F\xecS\xa6\x02H\x91\x08\x17\x9f\
~\x9a+;v\x10\x1d\x1c\xbcy\xfcJ\xecc\xd1\x9f\x896\xee{\x8b\x88\x05\x93\xb0\
\xaaR\xb5f\rjAm-U\r\r\xf4\xbc\xfb.!\x8b\xa1\x06\xe87np\xb9\xb9\x99\xa1O>!\
\xff\xb1\xc7\x08\xd5\xd4 gg\'\x80t/,b\x9b\x11\x8dr\xa3\xb77\xe1m\xc7l\xa2\
\x8d=\xf36\xf8<\xe0<P\xd5\xd0@Amm\xfc\x87n\xa9\xa3\x83J,_ \x96\x8fD\xad"[3*F\
\x03\xbb.\xbe-\x8bK\rL\x13\x0c#-\xc7\xf4\x8a6v=\xc3\x02\x9f\x0b\\\x02t\xe1Cw\
\xc2\xaf\x06\x91\x93\')#\xf6\x15\xd0\x06\xa7\xbb:\x16\x8f\xeeY\xf3Jyq]\x83D-\
\x88mv\x91\x81 1\x9b\x8f\x00W\x00\xd5\xebW\x03[\xec\x9f=:[[\tY?{\x04\x10\xc2\
\x16\xb3\xdf\xf4\x88Df\xfb\xac\x9d\xb2\x8c\x02c\x19\x19T\xd6\xd7\xb3\xe4\xa5\
\x97\x98sk\xa5\xd3\x95\x94\xeaw\x9b\xbe\xf6v&\xbb\xbb1S\xfcns3\xdb\xd0\xb4\
\xc44\xc1\xe7#X^NQ\x8a\xdfm\xfe\x07\\O\x9e\xe9m"?L\x00\x00\x00\x00IEND\xaeB`\
\x82\x94\x83\xda\x02'
