# Copyright (C) 2009 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/

import os
import re
import sys
import subprocess
import tempfile

import Image

if sys.platform.startswith('win'):
    _EXE        = '.exe'
else:
    _EXE        = ''
    
TEMP_DIR        = tempfile.gettempdir()
TEMP_RANDOM     = tempfile._RandomNameSequence()
TIFFINFO        = "tiffinfo"+_EXE
TIFFCP          = "tiffcp"+_EXE

RE_FIELD        = re.compile('\s+(.*?):\s+(.*?)\n')
RE_FIELD_IMAGE  = re.compile(' Image (.*?):\s+(.*?)$')


COMPRESSION     = {
    'CCITT Group 3'         : 'g3',
    'CCITT Group 4'         : 'g4',
    'Deflate'               : 'zip',
    'JPEG'                  : 'jpeg',
    'LZW'                   : 'lzw',
    'PackBits'              : 'packbits',
    'None'                  : 'none',
}

COMPRESSION_TYPES = COMPRESSION.values()
COMPRESSION_TYPES.sort()

def shell(*args):
    options = {'stdout':subprocess.PIPE,'stderr':subprocess.PIPE,
        'shell':True} #py2.5
    pipe    = subprocess.Popen(*args, **options)
    return pipe.stdout.read(), pipe.stderr.read()
  
def init(paths=[]):
    global TIFFINFO, TIFFCP
    for path in paths:
        if os.path.exists(path):
            files = os.listdir(path)
            if TIFFINFO in files and TIFFCP in files:
                TIFFINFO    = os.path.join(path, TIFFINFO)
                TIFFCP      = os.path.join(path, TIFFCP)
                return True
    if not sys.platform.startswith('win'):
        TIFFINFO, err   = shell('which '+TIFFINFO)
        TIFFCP, err     = shell('which '+TIFFCP)
        TIFFINFO        = TIFFINFO.strip()
        TIFFCP          = TIFFCP.strip()
        if TIFFINFO and TIFFCP:
            return True
    return False
    

def get_info(filename):
    result = {}
    def set(key, value):
        key = 'libtiff.'+key.lower().replace(' ','.')
        if not result.has_key(key):
            result[key] = value
    source, err = shell('%s %s'%(TIFFINFO,filename))
    for match in RE_FIELD.finditer(source):
        value   = match.group(2)
        again   = RE_FIELD_IMAGE.search(value)
        if again:
            set('Image %s'%again.group(1), again.group(2))
            set(match.group(1), value[:again.start()])
        else:
            set(match.group(1), value)
    if not result:
        raise IOError('Not a TIFF or MDI file, bad magic number.')
    result['compression']   = \
        COMPRESSION[result['libtiff.compression.scheme']]
    return result
    
def open(filename):
    """Opens a tiff file with tiffcp."""
    # get info
    info    = get_info(filename)
    # extract
    temp    = tempfile.NamedTemporaryFile().name
    shell('%s -c none -r -1 %s %s'%(TIFFCP,filename,temp))
    if os.path.exists(temp):
        image   = Image.open(temp)
        image.load() # otherwise temp file can't be deleted
        os.remove(temp)
        image.info.update(info)
        image.info['module'] = 'libtiff'
    else:
        raise IOError('Could not extract tiff image with tiffcp.')
    return image
    
def save(image, filename, compression=None, **options):
    if compression in ['raw','none']:
        image.save(filename,'tiff',**options)
        return ''
    if compression is None:
        compression = image.info['compression']
    elif compression in ['g3','g4'] and image.mode != '1':
        image   = image.convert('1')
    if compression == 'jpeg':
        if image.mode == 'RGBA':
            image   = image.convert('RGB')
        option = ' -r 16'
    else:
        option = ''
    temp    = tempfile.NamedTemporaryFile().name
    image.save(temp,'tiff',**options)
    out,err = shell('%s %s -c %s%s %s'%(TIFFCP,temp,compression,option,filename))
    os.remove(temp)
    if out or err:
        return 'Subprocess "tiffcp" output:\n%s%s\n'%(out,err)
    return ''
    
        
if __name__ == '__main__':
    print('Libtiff-tools installed: %s'%init())
    print COMPRESSION_TYPES
    filename    = '/home/stani/Downloads/0009.tif'
    image = open(filename)
    print(image.info)
    save(image.rotate(10),filename+'.tif',compression='g4')
